#    Copyright 2021 Qruise project
#
#    Licensed under the Apache License, Version 2.0 (the "License");
#    you may not use this file except in compliance with the License.
#    You may obtain a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS,
#    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#    See the License for the specific language governing permissions and
#    limitations under the License.

from qcalibrateremote._grpc import combine_key_and_secret, split_key_and_secret
from cuid import cuid
from icecream import ic

def test_combine_and_split():
    key = cuid()
    secret = cuid()
    ic(key, secret)
    
    keyAndSecret = combine_key_and_secret(client_key=key, client_secret=secret)
    ic(keyAndSecret)
    (k,s) = split_key_and_secret(keyAndSecret)
    assert key == k
    assert secret == s
